#!/bin/sh
#
# arch_is_concerned() tests
#
# Copyright (C) 2014  Patrick "P. J." McDermott
#
# This file is part of the ProteanOS Development Kit.
#
# The ProteanOS Development Kit is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# The ProteanOS Development Kit is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with the ProteanOS Development Kit.  If not, see
# <http://www.gnu.org/licenses/>.

. "${srcdir}/tests/common.sh"

use archplat

do_test()
{
	local host_arch="${1}"
	local arches="${2}"
	local result="${3}"

	if arch_is_concerned "${host_arch}" "${arches}"; then
		if ! ${result}; then
			printf 'False positive:\n'
			printf '  Host architecture:     "%s"\n' "${host_arch}"
			printf '  Package architectures: "%s"\n' "${arches}"
			tests_es=1
		fi
	else
		if ${result}; then
			printf 'False negative:\n'
			printf '  Host architecture:     "%s"\n' "${host_arch}"
			printf '  Package architectures: "%s"\n' "${arches}"
			tests_es=1
		fi
	fi
}

# Architecture: all

do_test 'amd64-linux-glibc' 'all' false

do_test 'i686-linux-glibc' 'all' false

do_test 'all' 'all' true

# Architecture: any

do_test 'amd64-linux-glibc' 'any' true

do_test 'i686-linux-glibc' 'any' true

do_test 'all' 'any' false

# Architecture: amd64-linux-glibc

do_test 'amd64-linux-glibc' 'amd64-linux-glibc' true

do_test 'i686-linux-glibc' 'amd64-linux-glibc' false

do_test 'all' 'amd64-linux-glibc' false

# Architecture: !amd64-linux-glibc

do_test 'amd64-linux-glibc' '!amd64-linux-glibc' false

do_test 'i686-linux-glibc' '!amd64-linux-glibc' true

do_test 'all' '!amd64-linux-glibc' false

# Architecture: amd64-linux-glibc cortexa8-linux-glibc

do_test 'amd64-linux-glibc' 'amd64-linux-glibc cortexa8-linux-glibc' true

do_test 'i686-linux-glibc' 'amd64-linux-glibc cortexa8-linux-glibc' false

do_test 'all' 'amd64-linux-glibc cortexa8-linux-glibc' false

# Architecture: amd64-linux-glibc all

do_test 'amd64-linux-glibc' 'amd64-linux-glibc all' true

do_test 'i686-linux-glibc' 'amd64-linux-glibc all' false

do_test 'all' 'amd64-linux-glibc all' true
